@@ -59,6 +59,8 @@
 	 (fh-load "xmessage.scm")
 	 (if (= (system "dict gnu >> /dev/null 2>&1") 0)
 	     (fh-load "dict-buddy.scm"))
+	 (if (= (system "beep -f 1 -l 0 >> /dev/null 2>&1") 0)
+	     (fh-load "beep.scm"))
 	 (add-hook! fh-login-post-hook delayed-loading))
        (lambda (k args . opts)
 	 (display "\n~qp~_~qp~_~qp~_~qp~_~qp~_~qp~")
