@@ -43,7 +43,7 @@
   (set! args (list->strlist args))
   (if (> (length args) 0)
       (begin
-	(fh-set-current-target-buddy! "shell")
+	(fh-set-current-target-buddy! "shell" "send")
 	(system (list->asv args " ")))))
 
 (fh-register-command! '("shell" "shell [OPTIONS]\n\t- ask shell buddy"))
