@@ -1,4 +1,4 @@
-/* 
+/*
     fh_utils.c: all utility/misc functions
 
     This program is free software; you can redistribute it and/or
@@ -41,27 +41,29 @@ cc_t old_vtime;
 struct termios term;
 
 static char fh_logo[] =
-  "  +-----------------------------------------------------+ \n"
-  "  |                                                     |+\n"
-  "  |                 ...   ...                           ||\n"
-  "  |               ..         ..                         ||\n"
-  "  |              ..          ..                         ||\n"
-  "  |               ...~~`'~~...                          ||\n"
-  "  |                '(O##(O).***                         ||\n"
-  "  |                  |##......*******......-_           ||\n"
-  "  |                  |##......................          ||\n"
-  "  |                  ##./ `.....FreeHoo.........        ||\n"
-  "  |                 (--)  `.................   ..       ||\n"
-  "  |                  ##   `.................     **     ||\n"
-  "  |                        .............. .       **    ||\n"
-  "  | Thank You for choosing .....    v .. ..        `*   ||\n"
-  "  | Free Software          `. ..     ......             ||\n"
-  "  | Please tell your        ....      .. ..             ||\n"
-  "  | friends about GNU       ....      .....             ||\n"
-  "  | <www.gnu.org>           WW WW      WW WW            ||\n"
-  "  |                                                     ||\n"
-  "  +-----------------------------------------------------+|\n"
-  "   +-----------------------------------------------------+\n";
+  "                                                           \n"
+  "              ...   ...       ----------------------       \n"
+  "            ..         ..    (  http://www.gnu.org  )      \n"
+  "           ..          ..     ----------------------       \n"
+  "            ...~~`'~~...                                   \n"
+  "             '(0##(0).***                                  \n"
+  "               |##......*******......-_                    \n"
+  "               |##......................                   \n"
+  "               ##./ `.....Freehoo.........                 \n"
+  "              (--)  `.................   ..                \n"
+  "               ##   `.................     **              \n"
+  "                     .............. .       **             \n"
+  "                     .....    v .. ..        `*            \n"
+  "                     `. ..     ......                      \n"
+  "                      ....      .. ..                      \n"
+  "                      ....       .. ..                     \n"
+  "                      WW WW      WW WW                     \n"
+  "     @=================================================@   \n"
+  "       Thank you for choosing Freehoo, but Jabber is a     \n"
+  "       better alternative and open standards based.        \n"
+  "                                  -- Freehoo Core Team     \n"
+  "     @=================================================@   \n"
+  "     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   \n";
 
 
 char *
@@ -71,7 +73,7 @@ get_fh_logo ()
 }
 
 /* handle SIGTERM or like signals and do a proper logout */
-void 
+void
 signal_handler (int signum)
 {
   /* set the quit flag for proper logout and freeing of resources */
@@ -294,12 +296,12 @@ filter_message (char *message)
   char *color_parsedstring = NULL;
   char *font_parsedstring = NULL;
   char *control_char_parsedstring = NULL;
-  
+
   color_parsedstring = parse_color_code (message);
 
-  font_parsedstring = (color_parsedstring) ? 
+  font_parsedstring = (color_parsedstring) ?
     parse_font_tag (color_parsedstring) : parse_font_tag (message);
-  
+
 
   if (color_parsedstring)
     free (color_parsedstring);
@@ -404,7 +406,7 @@ char *
 get_downloads_directory (void)
 {
   char *downloads_directory;
-  int length = 
+  int length =
     strlen (get_config_directory ()) + strlen (FH_DOWNLOADS_DIRECTORY) + 2;
 
   downloads_directory = (char *) calloc (length, sizeof (char));
