@@ -32,6 +32,7 @@
 
 #include <glib.h>
 #include <yahoo2.h>
+#include <regex.h>
 
 #include "freehoo.h"
 #include "interpreter.h"
@@ -48,6 +49,61 @@ char *commands[] = { "?send", "?who", "?add", "?remove", "?reject", "?ignore",
 		     "?send-file", FH_ROBOT, NULL
 };
 
+char * autocomplete_buddy_regex =  		
+                /* regex header */
+		"^\\ *\\?("
+		
+               /* commands which accept 1st arg as buddy */
+		"send|send\\-file|ignore|unignore|add|remove|reject|color\\-buddy|ping|buzz|forward|vconf\\-start|xmessage|"
+		
+		/* commands which accpet 1st arg and beyond as buddies */
+		"(forward|vconf\\-start)\\ +[a-zA-Z0-9_\\-\\ ]*|" 
+		
+		/* commands which accept 2nd arg and beyond
+                 * as buddies, but first as non buddy.. like
+                 * confroom or virtual buddy (not yet buddy) 
+                 */
+		"(alias|conf\\-start)\\ +[a-zA-Z0-9_\\-\\ ]+|"
+
+		/* command which accept 2nd arg only as 
+                 * buddy, 1st arg being non buddy, like 
+		 * confroom
+		 */
+		"(conf\\-add)\\ +[a-zA-Z0-9_\\-]+"
+
+		/* regex footer, possibly along with the 
+		 * half-d00d! at the end */
+		")\\ +[a-zA-Z0-9_\\-]*$" ;
+
+char * autocomplete_conf_regex = 		
+                /* regex header */
+		"^\\ *\\?("
+		
+               /* commands which accept 1st arg as confroom*/
+		"conf\\-add|conf\\-join|conf\\-decline|conf\\-end|conf\\-send"
+	
+		/* regex footer, possibly along with the 
+		 * half conf-room*/
+		")\\ +[a-zA-Z0-9_\\-]*$" ;
+
+char * autocomplete_file_regex = 		
+                /* regex header */
+		"^\\ *\\?("
+		
+               /* commands which accept 1st arg as filename*/
+		"send\\-file"
+		
+		/* buddy name */
+		")\\ +[a-zA-Z0-9_\\-]+\\ +" 
+
+		/* regex footer, possibly along with the 
+		 * half filename*/
+		"[a-zA-Z0-9_\\-\\/\\.]*$" ;
+
+char * autocomplete_cmd_regex = 		
+                /* still in first word or no words yet*/
+		"^\\ *\\??[a-zA-Z0-9_\\-]*$" ;
+
 extern YList *conferences;
 
 SCM dynamic_commands = SCM_UNSPECIFIED;
@@ -200,7 +256,7 @@ get_nth_buddy(int index)
 }
 
 char *
-command_generator (const char *text, int stat)
+command_generator (const char *text, int state)
 {
   static int i, len;
   static int buddy_index, toggle_index;
@@ -209,29 +265,95 @@ command_generator (const char *text, int stat)
   char *dynamic_command;
   char *name;
   yahoo_account *temp_buddy;
+  regex_t preg;
+  char curr_char = rl_line_buffer[rl_point];
+  conf_room *rooms;
+  YList *conflist = NULL;
+  static char complete_buddy = 0,
+	      complete_conf = 0,
+	      complete_toggle = 0,
+	      complete_file = 0,
+              complete_cmd = 0; // flags because regex is expensive
 
-  if (!stat)
+  if (!state)
     {
       len = strlen (text);
       i = 0;
       buddy_index   = 0;
       toggle_index  = 0;
       dynamic_cmd_index = 0;
-    }
+      complete_buddy = 0;
+      complete_conf = 0;
+      complete_toggle = 0;
+      complete_cmd = 0;
+      complete_file = 0;
+      conflist = get_fh_conferences();
 
-  // if your command takes buddy_name as argument then you
-  // will have to add ur command in this check list
-  if ((strncmp (rl_line_buffer, "?remove ", 8) == 0)
-      || (strncmp (rl_line_buffer, "?send ", 6) == 0)
-      || (strncmp (rl_line_buffer, "?reject ", 8) == 0)
-      || (strncmp (rl_line_buffer, "?add ", 5) == 0)
-      || (strncmp (rl_line_buffer, "?ignore ", 8) == 0)
-      || (strncmp (rl_line_buffer, "?unignore ", 10) == 0))
-    {
+     
+      rl_line_buffer[rl_point] = '\0'; /* the effect should be such
+					 * that the cursor position
+					 * is at the end of line for
+					 * the auto completion regex
+					 * above (note the $ at end)
+					 */
 
-      while((temp_buddy = (yahoo_account *) get_nth_buddy (i)))
+      /* check for the d00d's need in the context */
+      regcomp(&preg,autocomplete_buddy_regex,REG_EXTENDED|REG_ICASE);
+      if (  !regexec(&preg,rl_line_buffer,0,NULL,0))
+        {
+            complete_buddy = 1;
+        }
+       regfree(&preg);
+
+      /* check for a conf need in the context */
+      regcomp(&preg,autocomplete_conf_regex,REG_EXTENDED|REG_ICASE);
+      if ( !regexec(&preg,rl_line_buffer,0,NULL,0))
+        {
+            complete_conf = 1;
+        }
+       regfree(&preg);
+
+       /* check for a file need in the context */
+      regcomp(&preg,autocomplete_file_regex,REG_EXTENDED|REG_ICASE);
+      if ( !regexec(&preg,rl_line_buffer,0,NULL,0))
+        {
+            complete_file = 1;
+        }
+       regfree(&preg);
+     
+        /* ?toggle is pardoned for now	*/
+        if (strncmp (rl_line_buffer, "?toggle ", 8) == 0)
 	{
-	  i++;
+            complete_toggle = 1;
+	}
+	
+        regcomp(&preg,autocomplete_cmd_regex,REG_EXTENDED|REG_ICASE);
+        if ( !regexec(&preg,rl_line_buffer,0,NULL,0))
+        {
+            complete_conf = 1;
+            complete_cmd = 1;
+            complete_buddy = 1;
+        }
+        regfree(&preg);
+        
+	/* if no context available, default to buddy completion */
+	if(!complete_buddy && !complete_cmd && !complete_conf && !complete_cmd && !complete_file && !complete_toggle) 
+        {
+          /* complete_buddy = 1; */
+        }
+
+       rl_line_buffer[rl_point] = curr_char; /* what effect? :-) */
+
+      }
+
+    if(complete_file) 
+	return rl_filename_completion_function(text,state);
+
+    if(complete_buddy) 
+    {
+      while((temp_buddy = (yahoo_account *) get_nth_buddy (buddy_index)))
+	{
+	  buddy_index++;
 	  name = temp_buddy->yahoo_id;
 	  if (!name || (name[0] == 0))
 	    continue;
@@ -239,12 +361,22 @@ command_generator (const char *text, int stat)
 	  if (strncasecmp (name, text, len) == 0)
 	    return strdup (name);
 	}
-      
-      return NULL;
     }
-  
-  // ?toggle argument tabs
-  if (strncmp (rl_line_buffer, "?toggle ", 8) == 0)
+
+    if ( complete_conf)
+    {
+       /* conf rooms */
+       if(conflist) 
+       {
+         rooms = (conf_room *)conflist->data;
+         if(rooms && rooms->room_name && 
+             !strncasecmp(rooms->room_name,text,len))
+            return strdup(rooms->room_name);
+         conflist = conflist->next;
+       }
+    }
+
+    if (complete_toggle)
     {
       switch (toggle_index)
 	{
@@ -265,16 +397,17 @@ command_generator (const char *text, int stat)
 	  if (strncasecmp ("status", text, len) == 0)
 	    return strdup ("status");
 	default:
-	  return NULL;
+	  break;
+	  /* return NULL; */ // fallthrough needed now
 	}
     }
 
-  // for dynamic commands
-  if (gh_list_p (dynamic_commands))
-    {
-      dynamic_commands_count = gh_length (dynamic_commands);
-      while (dynamic_cmd_index < dynamic_commands_count)
-	{
+    if(complete_cmd) {
+       if (gh_list_p (dynamic_commands))
+       {
+          dynamic_commands_count = gh_length (dynamic_commands);
+          while (dynamic_cmd_index < dynamic_commands_count)
+          {
 	  dynamic_command = 
 	    gh_scm2newstr (gh_list_ref (gh_list_ref (dynamic_commands, 
 					gh_ulong2scm (dynamic_cmd_index++)),
@@ -283,29 +416,18 @@ command_generator (const char *text, int stat)
 	  if (dynamic_command && strncasecmp (dynamic_command, text, len) == 0)
 	    return strdup (dynamic_command);
 	}
+      }
+  
+      // for normal command
+      while ((name = commands[i]))
+        {
+          i++;
+          if (strncasecmp (name, text, len) == 0)
+	   return (char *) strdup (name);
+        }
     }
   
-  // for normal command
-  while ((name = commands[i]))
-    {
-      i++;
-      if (strncasecmp (name, text, len) == 0)
-	return (char *) strdup (name);
-    }
-
-  while((temp_buddy = (yahoo_account *) get_nth_buddy (buddy_index)))
-    {
-      buddy_index++;
-      
-      name = temp_buddy->yahoo_id;
-      if (!name || (name[0] == 0))
-	continue;
-
-      if (strncasecmp (name, text, len) == 0)
-	return strdup (name);
-    }
-  
-  return (char *) NULL;
+    return (char *) NULL; /* in the end */
 }
 
 char **
@@ -352,6 +474,21 @@ interpreter (char *line)
   
   if (strlen (line) == 0)
     {
+      /* AUTO-INSERT MODE and READLINE handling is kind of hack in
+      Freehoo. But it works great. If you cannot follow the source
+      code, please ask Anand Avati or Bala or Anand Babu */
+
+      current_target_buddy_mode_t mode;
+      static int recv_new;
+      get_current_target_buddy (&mode);
+      if ((mode == current_target_buddy_mode_send) ||
+	  (recv_new == 1))
+	{
+	  set_current_target_buddy( "",current_target_buddy_mode_send);
+	  recv_new = 0;
+	}
+      else
+	recv_new = 1;
       return;
     }
   
@@ -872,7 +1009,7 @@ command_send_message (char *line)
 	{
 	  yahoo_conference_message (get_fh_session ()->id,
 				    NULL, cr->members, user_id, line, 0);
-	  set_current_target_buddy (user_id);
+	  set_current_target_buddy (user_id,current_target_buddy_mode_send);
 	}
       else
 	send_message (user_id, line);
@@ -1157,7 +1294,7 @@ command_default_handler (char *command, char *line)
     {
       yahoo_conference_message (get_fh_session ()->id,
 				NULL, cr->members, command, line, 0);
-      set_current_target_buddy (command);
+      set_current_target_buddy (command,current_target_buddy_mode_send);
     }
   else
     {
@@ -1247,7 +1384,7 @@ command_conf_begin (char *line)
   yahoo_conference_invite (get_fh_session ()->id, NULL,
 			  cr->members, cr->room_name, "Join my conference");
   // AUTO-INSERT-MODE
-  set_current_target_buddy (cr->room_name);
+  set_current_target_buddy (cr->room_name,current_target_buddy_mode_send);
 }
 
 void
@@ -1277,7 +1414,7 @@ command_conf_quit (char *line)
     }
   free (cr);
   // AUTO-INSERT-MODE
-  set_current_target_buddy ("");
+  set_current_target_buddy ("",current_target_buddy_mode_send);
 }
 
 void
@@ -1312,8 +1449,8 @@ command_conf_decline (char *line)
     }
   free (cr);
   // AUTO-INSERT-MODE
-  if (strcmp (get_current_target_buddy (), room) == 0)
-    set_current_target_buddy ("");
+  if (strcmp (get_current_target_buddy (NULL), room) == 0)
+    set_current_target_buddy ("",current_target_buddy_mode_send);
 }
 
 void
@@ -1343,7 +1480,7 @@ command_conf_join (char *line)
   yahoo_conference_logon (get_fh_session ()->id, NULL,
 			 cr->members, room);
   // AUTO-INSERT-MODE
-  set_current_target_buddy (room);
+  set_current_target_buddy (room,current_target_buddy_mode_send);
 }
 
 void
@@ -1366,23 +1503,57 @@ command_conf_send (char *line)
   yahoo_conference_message (get_fh_session ()->id,
 			    NULL, cr->members, room, msg, 0);
   // AUTO-INSERT-MODE
-  set_current_target_buddy (room);
+  set_current_target_buddy (room,current_target_buddy_mode_send);
+}
+
+void send_file_getfd_callback(int session_id, int fd, int error, void *data)
+{
+  char buf[4096];
+  int bytes_read;
+  send_file_callback_data_t *cbk_data = (send_file_callback_data_t *)data;
+  int lfd = cbk_data->fd;
+  int fsize = cbk_data->fsize;
+  char *filename = cbk_data->fname;
+  char *filepath = cbk_data->fpath;
+
+  if (error)
+    {
+      printf ("Error in file [%s] upload\n", filename) ;
+      return;
+    }
+
+  while (fsize)
+    {
+      if((bytes_read = read (lfd, buf, sizeof(buf))) < 0)
+	{
+	  printf ("Error reading file [%s]\n", filename) ;
+	  break;
+	}
+      fsize -= bytes_read;
+      if (!bytes_read)
+	break;
+      if(write (fd, buf, bytes_read) != bytes_read)
+	{
+	  printf ("Error in file [%s] upload\n", filepath) ;
+	  break;
+	}
+    }
+  close (lfd);
+  printf ("File [%s] uploaded\n", filepath) ;
 }
 
 void
 command_send_file (char *line)
 {
-  /* FIXME */
-  printf ("Not implemented in this version\n");
-  return;
   
-  /*  char *user_id = NULL;
+  char *user_id = NULL;
   char *filepath = NULL;
   char *msg = NULL;
   struct stat file_stat;
-  int fd, fsize, lfd, bytes_read;
-  char filename[256], buf[4096];
+  int fsize, lfd;
+  char filename[256];
   char *ptr;
+  send_file_callback_data_t data; /* the callback is synchronous, so stack can be used for passing state, until libyahoo2 changes this in the future .. lets avoid global variable till then */
 
   user_id = get_token (&line);
   if (user_id == (char *) NULL)
@@ -1406,39 +1577,29 @@ command_send_file (char *line)
   if (!msg)
     msg = "";
   strcpy (filename, ptr);
-  fd = yahoo_send_file((get_fh_session ())->session_id, user_id, msg,
-		       filename, fsize);
-  if (fd < 0)
-    {
-      printf ("Error in file [%s] download\n", filename) ;
-      return;
-    }
 
+  /* this step better be done before yahoo_send_file() is called */
   lfd = open (filepath, O_RDONLY);
   if (lfd < 0)
     {
       printf ("Error in opening file [%s]\n", filepath) ;
       return;
     }
-  while (fsize)
-    {
-      if((bytes_read = read (lfd, buf, sizeof(buf))) < 0)
-	{
-	  printf ("Error reading file [%s]\n", filename) ;
-	  break;
-	}
-      fsize -= bytes_read;
-      if (!bytes_read)
-	break;
-      if(write (fd, buf, bytes_read) != bytes_read)
-	{
-	  printf ("Error in file [%s] upload\n", filepath) ;
-	  break;
-	}
-      
-    }
-  close (lfd);
-  close (fd);
-  printf ("File [%s] uploaded\n", filepath) ;
-  */
+
+  data.fd = lfd;
+  data.fsize = fsize;
+  data.fname = filename;
+  data.fpath = filepath;
+  /* last argument is a private callback data arg.. we use it by passing 
+   * current state (open fd and fsize)
+   */
+  yahoo_send_file((get_fh_session ())->id, user_id, msg,
+		       filename, fsize, send_file_getfd_callback,
+                       (void *)&data);
+}
+
+char *
+complete_none (const char *text,int state)
+{
+    return (char *)NULL;
 }
