@@ -27,6 +27,8 @@ unsigned char get_who_state (void);
 unsigned char get_session_mode (void);
 unsigned char get_status_mode (void);
 
+typedef struct { int fd, fsize; char *fname,*fpath; } send_file_callback_data_t;
+
 void toggle_bell (void);
 void toggle_who (void);
 void toggle_session (void);
@@ -45,11 +47,13 @@ char *command_generator (const char *text, int stat);
 char **complete_text (char *text, int start, int end);
 void syntax_error (void);
 void interpreter (char *line);
+char *complete_none(const char *text,int state);
 
 void command_show_help (char *line);
 void command_status (char *line);
 void command_send_message (char *line);
 void command_send_file (char *line);
+void send_file_getfd_callback(int id, int fd, int error, void *data);
 void command_buddy_add (char *line);
 void command_buddy_remove (char *line);
 void command_buddy_reject (char *line);
