@@ -190,23 +190,26 @@ ex_set_default_password_x (SCM scm_default_password)
 }
 
 SCM
-ex_set_current_target_buddy_x (SCM scm_current_target_buddy)
+ex_set_current_target_buddy_x (SCM scm_current_target_buddy, SCM scm_current_target_buddy_mode)
 {
   char *current_target_buddy = NULL;
-  if (scm_current_target_buddy == SCM_BOOL_F)
-    {
-      set_current_target_buddy (NULL);
-    }
-  else
+  current_target_buddy_mode_t mode = current_target_buddy_mode_send; // default to unset-able mode
+  
+  if (scm_current_target_buddy != SCM_BOOL_F)
     {
       current_target_buddy = 
 	gh_scm2newstr (scm_current_target_buddy, NULL);
-      
-      if (current_target_buddy[0] == '\0')
-	set_current_target_buddy (NULL);
-      else
-	set_current_target_buddy (current_target_buddy);
     }
+  if (scm_current_target_buddy_mode != SCM_BOOL_F)
+    {
+       if(!strcasecmp (gh_scm2newstr (scm_current_target_buddy_mode,NULL),
+			       "receive"))
+          {
+             mode = current_target_buddy_mode_receive;
+          }
+    }
+      
+  set_current_target_buddy (current_target_buddy,mode);
   return SCM_UNSPECIFIED;
 }
 
@@ -252,11 +255,11 @@ ex_get_buddy_list ()
 {
   int i = 0;
   SCM scm_buddy_list;
-  struct yahoo_buddy *temp_buddy;
+  yahoo_account *temp_buddy;
 
   scm_buddy_list = SCM_UNSPECIFIED;
 
-  while ((temp_buddy = (struct yahoo_buddy *) get_nth_buddy(i)))
+  while ((temp_buddy = (yahoo_account *) get_nth_buddy(i)))
     {
       if (!temp_buddy)
 	break;
@@ -264,7 +267,7 @@ ex_get_buddy_list ()
       if (gh_list_p (scm_buddy_list) != 1)
 	{
 	  scm_buddy_list = 
-	    gh_list (gh_str02scm (temp_buddy->id), 
+	    gh_list (gh_str02scm (temp_buddy->yahoo_id), 
 		     SCM_UNDEFINED);
 	}
       else
@@ -272,7 +275,7 @@ ex_get_buddy_list ()
 	  scm_buddy_list = 
 	    gh_append2 (scm_buddy_list,
 			gh_list (gh_str02scm 
-				 (temp_buddy->id),
+				 (temp_buddy->yahoo_id),
 				 SCM_UNDEFINED));
 	}
       i++;
