@@ -33,7 +33,7 @@ SCM ex_get_download_filename (void);
 SCM ex_set_default_login_id_x (SCM scm_default_login_id);
 SCM ex_get_default_login_id (void);
 SCM ex_set_default_password_x (SCM scm_default_password);
-SCM ex_set_current_target_buddy_x (SCM scm_current_target_buddy);
+SCM ex_set_current_target_buddy_x (SCM scm_current_target_buddy, SCM mode);
 SCM ex_send_message (SCM scm_to, SCM scm_message);
 SCM ex_send_message_no_hook (SCM scm_to, SCM scm_message);
 SCM ex_register_command_x (SCM command);
