@@ -157,21 +157,28 @@ get_local_addresses ()
 {
   static char addresses[1024];
   char buff[1024];
-  char gateway[16];
+  char interface[16];
+  char popen_cmd[32];
   char *c = NULL;
   struct hostent *hn;
-  int i;
   FILE *f;
+
   f = popen ("netstat -nr", "r");
   if ((int) f < 1)
     goto IP_TEST_2;
   while (fgets (buff, sizeof (buff), f) != NULL)
     {
-      c = strtok (buff, " ");
+      c = strtok (buff, " "); /* dest */
       if ((strstr (c, "default") || strstr (c, "0.0.0.0")) &&
 	  !strstr (c, "127.0.0"))
 	{
-	  c = strtok (NULL, " ");
+	  c = strtok (NULL, " "); /* gateway */
+	  c = strtok (NULL, " "); /* mask */
+	  c = strtok (NULL, " "); /* flags */
+	  c = strtok (NULL, " "); /* mss */
+	  c = strtok (NULL, " "); /* window */
+	  c = strtok (NULL, " "); /* irtt */
+	  c = strtok (NULL, " "); /* iface */
 	  pclose (f);
 	  break;
 	}
@@ -182,23 +189,19 @@ get_local_addresses ()
   if (!c || !c[0])
     goto IP_TEST_2;
   
-  strncpy (gateway, c, 16);
+  strncpy (interface, c, 16);
+  interface[16] = '\0'; /* just in case */
+  if(interface[strlen(interface)-1] == '\n')
+    interface[strlen(interface)-1] = '\0';
 
-  for (i = strlen (gateway); gateway[i] != '.'; i--)
-    gateway[i] = 0;
-
-  gateway[i] = 0;
-
-  for (i = strlen (gateway); gateway[i] != '.'; i--)
-    gateway[i] = 0;
-
-  f = popen ("/sbin/ifconfig -a", "r");
+  sprintf(popen_cmd,"/sbin/ifconfig %s",interface);
+  f = popen (popen_cmd, "r");
   if ((int) f < 1)
     goto IP_TEST_2;
 
   while (fgets (buff, sizeof (buff), f) != NULL)
     {
-      if (strstr (buff, "inet") && strstr (buff, gateway))
+      if (strstr (buff, "inet"))
 	break;
     }
   pclose (f);
@@ -416,7 +419,7 @@ ext_yahoo_conf_message (int id, char *who, char *room, char *msg, int utf8)
   ext_yahoo_got_im (id, who, buffer, 0, 0, utf8);
 
   // AUTO-INSERT-MODE
-  set_current_target_buddy (room);
+  set_current_target_buddy (room,current_target_buddy_mode_receive);
 
   if (utf8)
     FREE (umsg);
@@ -634,7 +637,7 @@ ext_yahoo_got_im (int id, char *who, char *msg, long tm, int stat, int utf8)
   msg = filter_message (msg);
 
   // AUTO-INSERT-MODE
-  set_current_target_buddy (who);
+  set_current_target_buddy (who,current_target_buddy_mode_receive);
 
   // invoke fh-message-receive-hook
   set_hook_return (0);
@@ -1283,33 +1286,19 @@ ext_yahoo_connect_async (int id, char *host, int port,
   static void local_input_callback(int source);
 */
 
-void
-ext_yahoo_got_file (int id, char *who, char *url, long expires, char *msg,
-		    char *fname, unsigned long fesize)
+
+void get_file_geturl_callback(int id, int fd, int error, const char * _fname, unsigned long size, void *_data)
 {
+  get_file_geturl_data_t *data = _data;
+  char *url = data->url;
   char lfile[256], *ptr, *qtr;
   char buf[4096];
-  unsigned long fsize;
-  int bytes_read, fd = -1, lfd = -1;
+  unsigned long fsize = data->fsize;
+  char *fname = data->fname;
+  int bytes_read,  lfd = -1,size_known = 0;
   char filename[256] = "";
 
-  if (msg && msg[0])
-    {
-      PRINTF_MESSAGE ("[%s] has sent a file [%s] with mesg [%s]\n", who, url, msg);
-    }
-  else
-    {
-      PRINTF_MESSAGE ("[%s] has sent a file [%s]\n", who, url);
-    }
-
-  //////////////////////////////////////////////////////
-  /* FIXME: broke while moving to newer libyahoo2-0.7.2
-     please ask <ab@gnu.org.in>, he has comments on this
-  fd = yahoo_get_url_handle(id, url, filename, &fsize);
-  */
-  return;
-  //////////////////////////////////////////////////////
-
+  if(fsize) size_known = 1;
   if(fname)
     strcpy (filename, fname);
   else
@@ -1333,7 +1322,8 @@ ext_yahoo_got_file (int id, char *who, char *url, long expires, char *msg,
       *qtr = '\0';
     }
 
-  if (fd < 0)
+
+  if (error != 0)
     {
       PRINTF_MESSAGE ("Error in file [%s] download\n", filename) ;
       return;
@@ -1364,24 +1354,51 @@ ext_yahoo_got_file (int id, char *who, char *url, long expires, char *msg,
       PRINTF_MESSAGE ("Error in opening file [%s]\n", lfile) ;
       return;
     }
-  while (fsize)
+  while ((size_known && fsize) || !size_known)
     {
-      if((bytes_read = read (fd, buf, sizeof(buf))) < 0)
+      if ((bytes_read = read (fd, buf, sizeof(buf))) < 0)
         {
           PRINTF_MESSAGE ("Error in file [%s] download\n", filename) ;
           break;
         }
-      fsize -= bytes_read;
-      if (!bytes_read)
-        break;
-      if(write (lfd, buf, bytes_read) != bytes_read)
+      if (size_known)
+          fsize -= bytes_read;
+      if (!bytes_read && errno == EAGAIN)
+        continue;
+      if (write (lfd, buf, bytes_read) != bytes_read)
         {
           PRINTF_MESSAGE ("Error in writing file [%s]\n", lfile) ;
           break;
         }
+      if (!size_known && !bytes_read )
+	      break;
     }
   close (lfd);
   PRINTF_MESSAGE ("File saved as [%s]\n", lfile) ;
+
+}
+
+void
+ext_yahoo_got_file (int id, char *who, char *url, long expires, char *msg,
+		    char *fname, unsigned long fesize)
+{
+  if (msg && msg[0])
+    {
+      PRINTF_MESSAGE ("[%s] has sent a file [%s] with mesg [%s] (%lu bytes)\n", who, fname, msg,fesize);
+    }
+  else
+    {
+      PRINTF_MESSAGE ("[%s] has sent a file [%s] (%lu bytes)\n", who, fname,fesize);
+    }
+
+
+    get_file_geturl_data_t data;
+    data.who = who;
+    data.url = url;
+    data.fname = fname;
+    data.fsize = fesize;
+    data.msg = msg;
+    yahoo_get_url_handle(id,url,get_file_geturl_callback,&data);
 }
 
 void
@@ -1524,9 +1541,9 @@ main_loop ()
 	    {
 	      if (get_auto_insert_mode () == 1)
 		{
-		  if (*(get_current_target_buddy ()) != '\0')
+		  if (*(get_current_target_buddy (NULL)) != '\0')
 		    {
-		      rl_insert_text (get_current_target_buddy ());
+		      rl_insert_text (get_current_target_buddy (NULL));
 		      rl_insert_text (" ");
 		      rl_redisplay ();
 		    }
@@ -1643,6 +1660,7 @@ messenger_main (int argc, char *argv[])
 
   rl_callback_handler_install (get_default_prompt (), interpreter);
   rl_attempted_completion_function = (CPPFunction *) complete_text;
+  rl_completion_entry_function = complete_none;
 
   main_loop ();
   LOG (("Exited loop"));
@@ -1709,6 +1727,13 @@ get_fh_buddies ()
   return buddies;
 }
 
+YList *
+get_fh_conferences ()
+{
+  return conferences;
+}
+
+
 YList *
 get_fh_ignores ()
 {
